package orderservicelogic

import (
	"app-cloudep-order-server/gen_result/pb/order"
	mocksvc "app-cloudep-order-server/internal/mock/lib"
	model "app-cloudep-order-server/internal/model/mongo"
	"app-cloudep-order-server/internal/svc"
	"context"
	"errors"
	"testing"

	"github.com/stretchr/testify/assert"
	"go.mongodb.org/mongo-driver/mongo"
	"go.uber.org/mock/gomock"

	mockmodel "app-cloudep-order-server/internal/mock/model"
)

func TestCancelOrder(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()

	// 初始化 mock 依賴
	mockOrderModel := mockmodel.NewMockOrderModel(ctrl)
	mockValidate := mocksvc.NewMockValidate(ctrl)

	// 初始化服務上下文
	svcCtx := &svc.ServiceContext{
		OrderModel: mockOrderModel,
		Validate:   mockValidate,
	}

	// 測試數據
	cancelReq := &order.CancelOrderReq{
		BusinessId: "12345",
		Status:     1,
	}

	// 測試數據集
	tests := []struct {
		name      string
		input     *order.CancelOrderReq
		prepare   func()
		expectErr bool
	}{
		{
			name:  "成功取消訂單",
			input: cancelReq,
			prepare: func() {
				// 模擬驗證通過
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬更新狀態成功
				mockOrderModel.EXPECT().UpdateStatus(gomock.Any(), model.UpdateStatusReq{
					BusinessID: "12345",
					Status:     1,
				}).Return(&mongo.UpdateResult{MatchedCount: 1}, nil).Times(1)
			},
			expectErr: false,
		},
		{
			name:  "訂單更新狀態失敗",
			input: cancelReq,
			prepare: func() {
				// 模擬驗證通過
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬更新狀態失敗
				mockOrderModel.EXPECT().UpdateStatus(gomock.Any(), model.UpdateStatusReq{
					BusinessID: "12345",
					Status:     1,
				}).Return(nil, errors.New("update failed")).Times(1)
			},
			expectErr: true,
		},
		{
			name:  "驗證失敗",
			input: cancelReq,
			prepare: func() {
				// 模擬驗證失敗
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(errors.New("invalid input")).Times(1)
			},
			expectErr: true,
		},
	}

	// 執行測試
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			// 設置測試環境
			tt.prepare()

			// 初始化 CancelOrderLogic
			logic := CancelOrderLogic{
				svcCtx: svcCtx,
				ctx:    context.TODO(),
			}

			// 執行 CancelOrder
			resp, err := logic.CancelOrder(tt.input)

			// 驗證結果
			if tt.expectErr {
				assert.Error(t, err)
				assert.Nil(t, resp)
			} else {
				assert.NoError(t, err)
				assert.NotNil(t, resp)
			}
		})
	}
}
