package orderservicelogic

import (
	"app-cloudep-order-server/gen_result/pb/order"
	mocksvc "app-cloudep-order-server/internal/mock/lib"
	mockmodel "app-cloudep-order-server/internal/mock/model"
	"app-cloudep-order-server/internal/svc"
	"context"
	"errors"
	"testing"
	"time"

	"github.com/shopspring/decimal"
	"github.com/stretchr/testify/assert"
	"go.uber.org/mock/gomock"
)

func TestBuildCreateOrderReq(t *testing.T) {
	tests := []struct {
		name      string
		input     *order.CreateOrderReq
		expectErr bool
		expected  *createOrderReq
	}{
		{
			name: "所有必填字段存在，可選字段為空",
			input: &order.CreateOrderReq{
				BusinessId:  "B123",
				OrderType:   1,
				OrderStatus: 2,
				Brand:       "TestBrand",
				OrderUid:    "UID123",
				ReferenceId: "REF123",
				Count:       "10.5",
				OrderFee:    "0.5",
				Amount:      "100",
			},
			expectErr: false,
			expected: &createOrderReq{
				BusinessID:  "B123",
				OrderType:   1,
				OrderStatus: 2,
				Brand:       "TestBrand",
				OrderUID:    "UID123",
				ReferenceID: "REF123",
				Count:       decimal.RequireFromString("10.5"),
				OrderFee:    decimal.RequireFromString("0.5"),
				Amount:      decimal.RequireFromString("100"),
			},
		},
		{
			name: "包含可選字段",
			input: &order.CreateOrderReq{
				BusinessId:       "B123",
				OrderType:        1,
				OrderStatus:      2,
				Brand:            "TestBrand",
				OrderUid:         "UID123",
				ReferenceId:      "REF123",
				Count:            "10.5",
				OrderFee:         "0.5",
				Amount:           "100",
				ReferenceBrand:   ptr("OtherBrand"),
				WalletStatus:     ptrInt64(1),
				ThreePartyStatus: ptrInt64(2),
			},
			expectErr: false,
			expected: &createOrderReq{
				BusinessID:       "B123",
				OrderType:        1,
				OrderStatus:      2,
				Brand:            "TestBrand",
				OrderUID:         "UID123",
				ReferenceID:      "REF123",
				Count:            decimal.RequireFromString("10.5"),
				OrderFee:         decimal.RequireFromString("0.5"),
				Amount:           decimal.RequireFromString("100"),
				ReferenceBrand:   ptr("OtherBrand"),
				WalletStatus:     ptrInt64(1),
				ThreePartyStatus: ptrInt64(2),
			},
		},
		{
			name: "無效的十進制字段",
			input: &order.CreateOrderReq{
				BusinessId:  "B123",
				OrderType:   1,
				OrderStatus: 2,
				Brand:       "TestBrand",
				OrderUid:    "UID123",
				ReferenceId: "REF123",
				Count:       "invalid-decimal", // 無效的數值
			},
			expectErr: true,
		},
		{
			name: "空字符串的可選欄位應返回 nil",
			input: &order.CreateOrderReq{
				BusinessId:    "B123",
				OrderType:     1,
				OrderStatus:   2,
				Brand:         "TestBrand",
				OrderUid:      "UID123",
				ReferenceId:   "REF123",
				Count:         "10.5",
				OrderFee:      "0.5",
				Amount:        "100",
				ThirdPartyFee: ptr(""), // 空字符串
			},
			expectErr: false,
			expected: &createOrderReq{
				BusinessID:    "B123",
				OrderType:     1,
				OrderStatus:   2,
				Brand:         "TestBrand",
				OrderUID:      "UID123",
				ReferenceID:   "REF123",
				Count:         decimal.RequireFromString("10.5"),
				OrderFee:      decimal.RequireFromString("0.5"),
				Amount:        decimal.RequireFromString("100"),
				ThirdPartyFee: nil, // 空字符串應返回 nil
			},
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			result, err := buildCreateOrderReq(tt.input)
			if tt.expectErr {
				assert.Error(t, err)
			} else {
				assert.NoError(t, err)
				assert.Equal(t, tt.expected, result)
			}
		})
	}
}

func TestCreateOrder(t *testing.T) {
	ctrl := gomock.NewController(t)
	defer ctrl.Finish()

	// 初始化 mock 依賴
	mockOrderModel := mockmodel.NewMockOrderModel(ctrl)
	mockValidate := mocksvc.NewMockValidate(ctrl)

	// 初始化服務上下文
	svcCtx := &svc.ServiceContext{
		OrderModel: mockOrderModel,
		Validate:   mockValidate,
	}

	// 測試數據集
	tests := []struct {
		name      string
		input     *order.CreateOrderReq
		prepare   func()
		expectErr bool
	}{
		{
			name: "成功建立訂單",
			input: &order.CreateOrderReq{
				BusinessId:  "B123",
				OrderType:   1,
				OrderStatus: 2,
				Brand:       "TestBrand",
				OrderUid:    "UID123",
				ReferenceId: "REF123",
				Count:       "10.5",
				OrderFee:    "0.5",
				Amount:      "100",
			},
			prepare: func() {
				// 模擬驗證成功
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬資料庫插入成功
				mockOrderModel.EXPECT().Insert(gomock.Any(), gomock.Any()).Return(nil).Times(1)
			},
			expectErr: false,
		},
		{
			name: "建立訂單時驗證失敗",
			input: &order.CreateOrderReq{
				BusinessId:  "B123",
				OrderType:   1,
				OrderStatus: 2,
				Brand:       "TestBrand",
				OrderUid:    "UID123",
				ReferenceId: "REF123",
				Count:       "10.5",
				OrderFee:    "0.5",
				Amount:      "100",
			},
			prepare: func() {
				// 模擬驗證失敗
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(errors.New("validation failed")).Times(1)
			},
			expectErr: true,
		},
		{
			name: "插入資料庫失敗",
			input: &order.CreateOrderReq{
				BusinessId:  "B123",
				OrderType:   1,
				OrderStatus: 2,
				Brand:       "TestBrand",
				OrderUid:    "UID123",
				ReferenceId: "REF123",
				Count:       "10.5",
				OrderFee:    "0.5",
				Amount:      "100",
			},
			prepare: func() {
				// 模擬驗證成功
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬插入資料庫失敗
				mockOrderModel.EXPECT().Insert(gomock.Any(), gomock.Any()).Return(errors.New("insert failed")).Times(1)
			},
			expectErr: true,
		},
		{
			name: "成功建立訂單，必填字段",
			input: &order.CreateOrderReq{
				BusinessId:  "B123",
				OrderType:   1,
				OrderStatus: 2,
				Brand:       "TestBrand",
				OrderUid:    "UID123",
				ReferenceId: "REF123",
				Count:       "10.5",
				OrderFee:    "0.5",
				Amount:      "100",
			},
			prepare: func() {
				// 模擬驗證成功
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬資料庫插入成功
				mockOrderModel.EXPECT().Insert(gomock.Any(), gomock.Any()).Return(nil).Times(1)
			},
			expectErr: false,
		},
		{
			name: "成功建立訂單，可選字段有值",
			input: &order.CreateOrderReq{
				BusinessId:       "B123",
				OrderType:        1,
				OrderStatus:      2,
				Brand:            "TestBrand",
				OrderUid:         "UID123",
				ReferenceId:      "REF123",
				Count:            "10.5",
				OrderFee:         "0.5",
				Amount:           "100",
				ReferenceBrand:   ptr("OtherBrand"),
				WalletStatus:     ptrInt64(1),
				CryptoType:       ptr("BTC"),
				PaymentFiat:      ptr("USD"),
				OrderArrivalTime: ptrInt64(time.Now().UTC().Unix()),
			},
			prepare: func() {
				// 模擬驗證成功
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬資料庫插入成功
				mockOrderModel.EXPECT().Insert(gomock.Any(), gomock.Any()).Return(nil).Times(1)
			},
			expectErr: false,
		},
		{
			name: "部分可選字段有值，其他為空",
			input: &order.CreateOrderReq{
				BusinessId:     "B123",
				OrderType:      1,
				OrderStatus:    2,
				Brand:          "TestBrand",
				OrderUid:       "UID123",
				ReferenceId:    "REF123",
				Count:          "10.5",
				OrderFee:       "0.5",
				Amount:         "100",
				ReferenceBrand: ptr("OtherBrand"),
				WalletStatus:   ptrInt64(1),
				CryptoType:     nil, // 空
				PaymentFiat:    nil, // 空
			},
			prepare: func() {
				// 模擬驗證成功
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬資料庫插入成功
				mockOrderModel.EXPECT().Insert(gomock.Any(), gomock.Any()).Return(nil).Times(1)
			},
			expectErr: false,
		},
		{
			name: "無效的十進制數字",
			input: &order.CreateOrderReq{
				BusinessId:  "B123",
				OrderType:   1,
				OrderStatus: 2,
				Brand:       "TestBrand",
				OrderUid:    "UID123",
				ReferenceId: "REF123",
				Count:       "invalid", // 無效的數值
				OrderFee:    "0.5",
				Amount:      "100",
			},
			prepare: func() {
				// 無需調用資料庫操作，因為會在驗證或轉換時失敗
			},
			expectErr: true,
		},
		{
			name: "可選欄位為空字符串",
			input: &order.CreateOrderReq{
				BusinessId:    "B123",
				OrderType:     1,
				OrderStatus:   2,
				Brand:         "TestBrand",
				OrderUid:      "UID123",
				ReferenceId:   "REF123",
				Count:         "10.5",
				OrderFee:      "0.5",
				Amount:        "100",
				ThirdPartyFee: ptr(""), // 空字符串
			},
			prepare: func() {
				// 模擬驗證成功
				mockValidate.EXPECT().ValidateAll(gomock.Any()).Return(nil).Times(1)
				// 模擬資料庫插入成功
				mockOrderModel.EXPECT().Insert(gomock.Any(), gomock.Any()).Return(nil).Times(1)
			},
			expectErr: false,
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			// 設置測試環境
			tt.prepare()

			// 初始化 CreateOrderLogic
			logic := NewCreateOrderLogic(context.TODO(), svcCtx)

			// 執行 CreateOrder
			resp, err := logic.CreateOrder(tt.input)

			// 驗證結果
			if tt.expectErr {
				assert.Error(t, err)
				assert.Nil(t, resp)
			} else {
				assert.NoError(t, err)
				assert.NotNil(t, resp)
			}
		})
	}
}

// ================================ 輔助函數 ================================
func ptr(s string) *string {
	return &s
}

func ptrInt64(i int64) *int64 {
	return &i
}

// decimalPtr 是一個輔助函數，用來將字串轉換為 *decimal.Decimal 指針
func decimalPtr(val string) *decimal.Decimal {
	d, _ := decimal.NewFromString(val)
	return &d
}
